import random

from ._common import round2MSF
#from _common import round2MSF

# 위 import의 common 앞의 .(dot)은 반드시 필요하다.
# 본 코드를 수정하면, .(dot)을 삭제하고 실행한다.
# 그러나, 오류가 없음을 확인하면 반드시 .(dot)을 되돌린 후 저장한다.

# 보기문항은 반드시 정답란에서 정한다.
# 보기문항은 string object으로 생성되며 이름은 choices으로 한다.
# 즉, 이름이 choices인 변수가 존재하면,  선택형 질문이 된다.


def generateChoices(ans, nChoices: int=4): # generateEm.work.makeSheets()에서 사용
    '''
ans: either a number or a dictionary object
    when it is a number, either integer or floating number,
        choices and correct choice are generated randomly.

    when it is a dictionary object,
        a key of "choices" must be present with/without a value.
        the value of "ans" key should the correct answer value, not the index of it among the choices provided.

        if the value of "choices" key is None,
        choices and correct choice are generated randomly with the specified method, given by the value for "fn" key.

        ** a list of methods available can be found by running the code below in Shell screen **

            gNs.makeChoices.fns

        if the key of "cols" is present,
            it indicates a special layout; each choice is a figure.
            this case used multiple canvases in HTML page and a lot of extra coding in HTML, CSS, and Javascript is involved.
            ...
            this case requires more attention in future


To test this function, try the code below in Shell screen.

    gNs.makeChoices.generateChoices(ans)

'''

    columnsN=None
    if isinstance(ans, dict) and 'choices' in ans:

        if 'cols' in ans and ans['cols']: columnsN=ans['cols']

        if ans['choices'] and isinstance(ans['choices'], (list, tuple)):
            try:
                choices=ans['choices'] # 이 경우, 정답, ans, 은 항상 들어있음.
                ans=ans['ans']
            except Exception as err:
                return f'<font color="#dd0000"><bold>Error</bold></font>::  {err}', 0, 0
        elif 'fn' in ans:
            if ans['fn'] in fns:
                fn=eval(ans['fn'])
                ans=round2MSF(ans['ans'])
                choices=fn(str(ans))
            else:
                return '<font color="#dd0000"><bold>Error</bold></font>::  ??????', 0, 0
        else:
##            return f'<font color="#dd0000"><bold>Error</bold></font>::  {err}', 0, 0
            return '', -1, ans['ans'], None # 오류 대신 무조건 short-answer question으로 정함. -1 <---특별한 의미.
    else:
        if ans: # 정수 또는 실수인 일반적인 경우
            ans=round2MSF(ans)
            if isinstance(ans, int):
                if random.randint(0, 1):
                    choices=variation0_int(str(ans))
                else:
                    tmp=random.randint(0,3)
                    if tmp==2:
                        choices=variation0_int_5Add(str(ans)) if random.randint(0,1) else variation0_int(str(ans))
                    elif tmp==1:
                        choices=variation0_int_3Add(str(ans)) if random.randint(0,1) else variation0_int(str(ans))
                    else:
                        if ans > 50:
                            choices=variation0_int_2Add(str(ans)) if random.randint(0,1) else variation0_int(str(ans))
                        else:
                            if random.randint(0,3):
                                choices=variation0_int_2Add(str(ans)) if random.randint(0,1) else variation0_int(str(ans))
                            else:
                                choices=variation0_int_2(str(ans)) if random.randint(0,1) else variation0_int(str(ans))
            else:
                if random.randint(0,5):
                    choices=variation0(str(ans))
                else:
                    choices=variation2(str(ans))
        else:
            ans=0
            choices=variation0_int(str(ans))

    choices.remove(ans)
    random.shuffle(choices)
    if not random.choice(range(8)):
        if len(choices)<nChoices:
            print(len(choices), nChoices, ans, choices, '답 없음 ...')
            nChoices=len(choices)
        answers=random.sample(choices, nChoices)
        return _ChoicesWithRadiobuttons(answers, columnsN), nChoices+1, ans, answers # 4th 추가
    else:
        if len(choices)<nChoices-1:
            print(len(choices), nChoices-1, ans, choices, '답 ...')
            nChoices = len(choices)+1
        answers=random.sample(choices, nChoices-1)
        answers.append(ans)
        random.shuffle(answers)
        return _ChoicesWithRadiobuttons(answers, columnsN), answers.index(ans)+1, ans, answers # 4th 추가


def _ChoicesWithNumbers(answers):
    choices = '<br>'
    for j, choice in enumerate(answers):
        choices += '<br>'+chr(9312+j)+f' {choice}'

    choices += '<br>'+chr(9312+j)+' None of the above'
    choices += '<br>'+chr(9313+j)+' All of the above'
    return choices


##def _ChoicesWithRadiobuttons(answers):
##    choices = '<br> '
##    for j, choice in enumerate(answers):
##        charInCircle=chr(9312+j)
##        choices += f'<br><input type="radio" style="height:20px; width:20px; vertical-align: middle; border-radius: 50%; bckground: white;" id="{charInCircle}" name="QqNUM" onclick="handleClick(this);" value="{j}"><label for="{charInCircle}"> {choice}</label>'
##
##    charInCircle=chr(9312+j)
##    choices += f'<br><input type="radio" style="height:20px; width:20px; vertical-align: middle; border-radius: 50%; bckground: white;" id="{charInCircle}" name="QqNUM" onclick="handleClick(this);" value="{j+1}"><label for="{charInCircle}"> None of the above</label>'
##    charInCircle=chr(9313+j)
##    choices += f'<br><input type="radio" style="height:20px; width:20px; vertical-align: middle; border-radius: 50%; bckground: white;" id="{charInCircle}" name="QqNUM" onclick="handleClick(this);" value="{j+2}"><label for="{charInCircle}"> All of the above</label>'
##    return choices


def _ChoicesWithRadiobuttons(answers, columns):
    if columns:
        choices = '<div> '
        for j, choice in enumerate(answers+['None of the above', 'All of the above']):
            if 'above' in choice:
                choices += f'<div style="float: left"><label class="radiocontainer"> {choice}<input type="radio" name="QqNUM" onclick="handleClick(this);" value="{j}"><span class="checkmark"></span></label></div>'
            else:
                choices += f'<div style="float: left"><label class="radiocontainer"> <div id="cnvs{j}"></div><input type="radio" name="QqNUM" onclick="handleClick(this);" value="{j}"><span class="checkmark"></span></label></div>'

        return choices+'</div>'

    else:        
        choices = '<div> <div style="float: left">'
        for j, choice in enumerate(answers+['None of the above', 'All of the above']):
            choices += f'<label class="radiocontainer"> {choice}<input type="radio" name="QqNUM" onclick="handleClick(this);" value="{j}"><span class="checkmark"></span></label>'

        return choices+'</div></div>'

    
def variation0_int_5Add(answer: str):
    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=answer[1:] # 음수기호가 제거된 답

    for i4NZ, c in enumerate(ans[::-1]):
        if int(c):
            break
    if i4NZ:
        ans=ans[:-i4NZ]

    zeros='0'*i4NZ

    value = int(ans)
    if value<30:
        tmp = [f'{value+5*j}{zeros}' for j in range(-3,7) if len(f'{value+5*j}')<=3]
    else:
        tmp = [f'{value+5*j}{zeros}' for j in range(-4,6) if len(f'{value+5*j}')<=3]
     
    # 음수 복원
    if negative: tmp=['-'+item for item in tmp if item]

    return [eval(f) for f in tmp]

   
##def variation0_int_5(answer: str):
##    # 음수여부
##    ans=answer
##    negative=False
##    if answer.startswith('-'):
##        negative=True
##        ans=answer[1:] # 음수기호가 제거된 답
##
##    for i4NZ, c in enumerate(ans[::-1]):
##        if int(c):
##            break
##    if i4NZ:
##        ans=ans[:-i4NZ]
##
##    zeros='0'*i4NZ
##
##    # 약수 5의 갯수
##    value = int(ans)
##    nYaksoo=0
##    while not value%5:
##        value //= 5
##        nYaksoo += 1
##
##    tmp = [f'{value*5**j}{zeros}' for j in range(10) if len(f'{value*5**j}')<=3]
##     
##    # 음수 복원
##    if negative: tmp=['-'+item for item in tmp if item]
##
##    return [eval(f) for f in tmp]
##
##
##def variation0_int_3(answer: str):
##    # 음수여부
##    ans=answer
##    negative=False
##    if answer.startswith('-'):
##        negative=True
##        ans=answer[1:] # 음수기호가 제거된 답
##
##    for i4NZ, c in enumerate(ans[::-1]):
##        if int(c):
##            break
##    if i4NZ:
##        ans=ans[:-i4NZ]
##
##    zeros='0'*i4NZ
##
##    # 약수 3의 갯수
##    value = int(ans)
##    nYaksoo=0
##    while not value%3:
##        value //= 3
##        nYaksoo += 1
##
##    tmp = [f'{value*3**j}{zeros}' for j in range(10) if len(f'{value*3**j}')<=3]
##     
##    # 음수 복원
##    if negative: tmp=['-'+item for item in tmp if item]
##
##    return [eval(f) for f in tmp]
##
def variation0_int_3Add(answer: str):
    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=answer[1:] # 음수기호가 제거된 답

    for i4NZ, c in enumerate(ans[::-1]):
        if int(c):
            break
    if i4NZ:
        ans=ans[:-i4NZ]

    zeros='0'*i4NZ

    value = int(ans)
    if value<20:
        tmp = [f'{value+3*j}{zeros}' for j in range(-3,7) if len(f'{value+3*j}')<=3]
    else:
        tmp = [f'{value+3*j}{zeros}' for j in range(-4,6) if len(f'{value+3*j}')<=3]
     
    # 음수 복원
    if negative: tmp=['-'+item for item in tmp if item]

    return [eval(f) for f in tmp]


def variation0_int_2Add(answer: str):
    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=answer[1:] # 음수기호가 제거된 답

    for i4NZ, c in enumerate(ans[::-1]):
        if int(c):
            break
    if i4NZ:
        ans=ans[:-i4NZ]

    zeros='0'*i4NZ

    value = int(ans)
    if value<20:
        tmp = [f'{value+2*j}{zeros}' for j in range(-3,7) if len(f'{value+2*j}')<=3]
    else:
        tmp = [f'{value+2*j}{zeros}' for j in range(-4,6) if len(f'{value+2*j}')<=3]
     
    # 음수 복원
    if negative: tmp=['-'+item for item in tmp if item]

    return [eval(f) for f in tmp]


def variation0_int_2(answer: str):
    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=answer[1:] # 음수기호가 제거된 답

    for i4NZ, c in enumerate(ans[::-1]):
        if int(c):
            break
    if i4NZ:
        ans=ans[:-i4NZ]

    zeros='0'*i4NZ

    # 약수 2의 갯수
    value = int(ans)
    nYaksoo=0
    while not value%2:
        value //= 2
        nYaksoo += 1

    tmp = [f'{value*2**j}{zeros}' for j in range(10) if len(f'{value*2**j}')<=3]
     
    # 음수 복원
    if negative: tmp=['-'+item for item in tmp if item]

    return [eval(f) for f in tmp]



def variation0_int(answer: str):
    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=answer[1:] # 음수기호가 제거된 답
    
    if len(ans)==1:
        tmp = [str(j) for j in range(10)]
    elif len(ans) == 2:
        if random.randint(0,1):
            tmp = [ans[0]+str(j) for j in range(10)]
        else:
            tmp = [str(j)+ans[1] for j in range(1, 10)]
    else:
        whichone = random.randint(0, 2)
        if  whichone == 0:
            # 유효숫자들 중 첫 번째숫자를 1부터 9까지 변화시킨 목록 생성
            tmp=[f'{i}'+ans[1:] for i in range(1, 10)] # 2022.07.16 추가...
        elif whichone == 1:
            # 유효숫자들 중 2 번째를 0부터 9까지 변화시킨 목록 생성
            tmp=[ans[:1]+f'{i}'+ans[2:] for i in range(10)]
        else:
            # 유효숫자들 중 3 번째를 0부터 9까지 변화시킨 목록 생성
            tmp=[ans[:2]+f'{i}'+ans[3:] for i in range(10)]
 
    # 음수 복원
    if negative: tmp=['-'+item for item in tmp if item]

    return [eval(f) for f in tmp]


def variation0(answer: str):
    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=ans[1:] # 음수기호가 제거된 답

    # 소수점여부
    iDot=None
    if '.' in ans:
        iDot=ans.index('.')
        ans=''.join(ans.split('.')) #소수점이 제거된 답

    # i: 0이 아닌 숫자 index
    for i4NZ, c in enumerate(ans):
        if int(c): break
    
    if len(ans)==1:
        tmp=[ans[:i4NZ]+f'{i}'+ans[i4NZ+1:] for i in range(10)]
    else:
        if random.randint(0,1):
            # 유효숫자들 중 첫 번째숫자를 0부터 9까지 변화시킨 목록 생성
            tmp=[ans[:i4NZ]+f'{i}'+ans[i4NZ+1:] for i in range(10)] # 2022.07.16 추가...
        else:
            # 유효숫자들 중 2 번째를 0부터 9까지 변화시킨 목록 생성
            tmp=[ans[:i4NZ+1]+f'{i}'+ans[i4NZ+2:] for i in range(10)]

    # 소수점 복원
    if iDot: tmp=[item[:iDot]+'.'+item[iDot:] for item in tmp]
    # 음수 복원
    if negative: tmp=['-'+item for item in tmp if item]

    return [eval(f) for f in tmp]



def variation2(answer: str):

    # 음수여부
    ans=answer
    negative=False
    if answer.startswith('-'):
        negative=True
        ans=ans[1:] # 음수기호가 제거된 답

    # 소수점여부
    iDot=len(ans)
    if '.' in ans:
        iDot=ans.index('.')
        ans=''.join(ans.split('.')) #소수점이 제거된 답

    if 'e' in ans: # added 2023.09.02
        ansBeforeE, ansAfterE = ans.split('e')
        
        ans='0'*3+ansBeforeE+'0'*3 # 2022.07.16 수정..

        a=[(ans[:i]+'.'+ans[i:]).strip('0') for i in range(1, len(ans))]
        a=['0'+item if item.startswith('.') else item for item in a]
        ans=[item[:-1] if item.endswith('.') else item for item in a]
        if negative: ans=['-'+item for item in ans if item] # 음수 복원

        return [eval(f+'e'+ansAfterE) for f in ans]
    else:
        ans='0'*3+ans+'0'*3 # 2022.07.16 수정..

        a=[(ans[:i]+'.'+ans[i:]).strip('0') for i in range(1, len(ans))]
        a=['0'+item if item.startswith('.') else item for item in a]
        ans=[item[:-1] if item.endswith('.') else item for item in a]
        if negative: ans=['-'+item for item in ans if item] # 음수 복원

        return [eval(f) for f in ans]


fns=[k for k in globals().keys() if k.startswith('v')]
if __name__ == '__main__':
    for j in range(10000):
        #generateChoices(random.choice(range(10000))/10)
        generateChoices(random.choice(range(10000)))



import math, random, re, json
from urllib.request import urlopen
import urllib

def getFigure(file: str):
    try:
        return json.loads(urlopen(f'https://generateNscore.github.io/html4quiz/JS/{file}.json').read())
    except urllib.error.HTTPError:
        return 'HTTP Error 404: Not Found'


def getResource(src: str):
    try:
        return json.loads(urlopen(f'https://generateNscore.github.io/html4quiz/res/{src}.json').read())
    except urllib.error.HTTPError:
        return 'HTTP Error 404: Not Found'


def round2MSF(a):
    if not a or (isinstance(a, int) and abs(a)<1000) : return a

    sfN=3
    signFlag=False
    if a<0: a=-a; signFlag=True
    n=int(math.log10(a))
    if a>1: n +=1
    b=round(a*10**(-n), sfN) # floating number 0.1<b<1
    if signFlag: b *= -1
    
    ans=eval(f'{round(b,3)}e{n}')
    if str(ans).endswith('.0') or str(ans).endswith('.00'): ans=int(ans)
    return ans


def getStudList(csvName: str='', header4SID: str='', header4Name: str=''):
    '''Usage: getStudList('csvName.csv', 'ID', 'Name')'''
    import csv
    if not csvName or not csvName.endswith('.csv') or not csvName.split('.csv')[0]:
        return 'Error: no csv file name'

    if not header4SID:
        return 'Error: no column header for student IDs'

    if not header4Name:
        return 'Error: no column header for student names'

    try:
        with open(csvName) as csvFile:
            csvData=[row for row in csv.reader(csvFile, delimiter=',')]
    except UnicodeDecodeError:
        with open(csvName, encoding='UTF-8') as csvFile:
            csvData=[row for row in csv.reader(csvFile, delimiter=',')]
    except Exception as err:
        return f'Error: reading error, contact me, {err}'

    if not csvData:
        return 'Error: Failed reading in student list'
        
    Headers = [item.replace('\ufeff', '').strip() if item.startswith('\ufeff') else item.strip() for item in csvData[0]]
    if not all([len(item)==len(Headers) for item in csvData[1:]]):
        return 'Error: Incosistency in columns'

    if header4SID in Headers:
        iPIN=Headers.index(header4SID)
    else:
        return f'Error: no column for {header4SID}'
    
    if header4Name in Headers:
        iName=Headers.index(header4Name)
    elif f'\ufeff{header4Name}' in Headers:
        iName=Headers.index(f'\ufeff{header4Name}')
    else:
        return f'Error: no column for {header4Name}'

    listSN={item[iPIN].strip(): item[iName].strip() for item in csvData[1:] if item[iPIN].strip()}

##    for k,v in listSN.items():
##        if '\n' in v: listSN[k]=v.replace('\n','_')

    return listSN

if __name__=='__main__':
    print(getStudList('../출석부.csv','학번','학생명'))




import pickle, os, datetime

from ._common import round2MSF # scoreAnswerFiles.compare에서 사용.


class scoreAnswerFiles:

    def __init__(self, workPickle, path4files=None):
        self.ext='txt'
        self.path4files=path4files
        if workPickle and os.path.exists(workPickle):
            self.initWork(workPickle)
        else:
            print('We need the pickle fie')
            return

    def initWork(self, workPickle):
        self.work=pickle.load(open(workPickle,'rb'))
        self.WN=self.work.Name
        self.QGs=self.work.QGs
        self.Sheets=self.work.Sheets
        self.nProblems=len(self.QGs)

        if self.path4files and os.path.exists(self.path4files):
            try:
                self.getAnswersFromFiles()
            except Exception as err:
                print(err, '@ Ln25')
                return

            try:
                self.scoreThem()
            except Exception as err:
                print(err, '@ Ln31')
                return

            try:
                self.saveResults()
            except Exception as err:
                print(err, '@ Ln37')
                return
        else:
            print("we need the folder that students' answer text files are saved")
            return
                
            

    def getAnswersFromFiles(self):
        for r, v in enumerate(self.Sheets.values()):
            v['indexLB']=r # 필요함.
            v['file']='' # 답안지 파일 .pickle/.txt
            v['studAns']=[] # 정답은 v['Q&A'][1]에 있음.
            v['py']={} # 코드제출 python file 목록, 문제번호: (filename, func2bCalled object, 코드)
            v['msg']=''
            v['ox']=['X']*self.nProblems
            v['score']=0
            v['log']=[]

        self.getFiles() # just try to get the correct file names as ruled.
        self.readAnswers() # try to get the answer from the files


    def readAnswers(self):
        for sid, v in self.Sheets.items():
            if not v['file']: continue
            lines=self.txt2Answer(os.path.join(self.path4files,v['file'][0]))
            if isinstance(lines, str) and lines.startswith('readingError'):
                v['msg']=lines
                continue
            v['studAns']=lines

        for sid, v in self.Sheets.items():
            if not v['file']: continue
            if len(v['studAns'])<self.nProblems:
                v['studAns'].extend(['미제출']*(self.nProblems-len(v['Answer'])))
    


    def txt2Answer(self, txtPath):
        tenBytes=open(txtPath, 'rb').read(10)
        if b'\xff' in tenBytes:
            try:
                lines=open(txtPath, 'r', encoding='UTF-16-LE').readlines()
                lines=[line.split('\ufeff')[-1] if line.startswith('\ufeff') else line for line in lines]
            except:
                return 'readingError utf16LE'
        elif b'\xef' in tenBytes:
            try:
                lines=open(txtPath, 'r', encoding='UTF-8-SIG').readlines()
            except:
                return 'readingError utf8 bom'
        else:
            try:
                lines=open(txtPath, 'r', encoding='utf-8').readlines()
            except:
                try:
                    lines=open(txtPath, 'r', encoding='ansi').readlines()
                except:
                    return 'readingError UTF8/ANSI'

        answersFromTxt=['' for i in range(self.nProblems)]
        for line in lines:
            if not line.strip(): continue
            if '#' in line: continue
            if ':' not in line: continue
            hdr, *hdrValue=line.strip().split(':')
            hdr=hdr.strip()
            hdrValue=':'.join(hdrValue)
            if hdr.isnumeric() and int(hdr)<=self.nProblems and hdrValue:
                try:
                    answersFromTxt[int(hdr)-1]=eval(hdrValue)
                except SyntaxError: # 2023-04-14 added
                    answersFromTxt[int(hdr)-1]=hdrValue.strip() # 2023-04-14 added
                except Exception as err:
                    answersFromTxt[int(hdr)-1]=f'Error({err}) in ({hdrValue}) @ Ln90'

        return answersFromTxt


    def getFiles(self):
        fileList=[f for f in os.listdir(self.path4files) if 'Scores' not in f and f.endswith(self.ext)]
        mtime=[str(datetime.datetime.fromtimestamp(os.stat(self.path4files+f'/{f}').st_mtime))[5:] for f in fileList]
        sortingIndex=sorted(range(len(mtime)), key=mtime.__getitem__, reverse=True)

        for sid, v in self.Sheets.items():
            for j in sortingIndex:
                if sid in fileList[j] and self.WN in fileList[j]:
                    self.Sheets[sid]['file']=(fileList[j], mtime[j],
                                              os.stat(os.path.join(self.path4files,fileList[j])).st_size)
##                    if mtime[j]>late:
##                        self.Sheets[sid]['msg']='late(0)'
                    break

        for v in self.Sheets.values():
            if not v['file'] and not v['msg']:
                v['msg']='no submission(0)'

        
    def scoreThem(self):
        for sid, v in self.Sheets.items():
            if not v['file'] or v['msg']: continue
            for nChecked, (wv, ans0) in enumerate(zip(v['studAns'], v['Q&A'])):
                if isinstance(wv, str) and 'Error' in wv:
                    v['ox'][nChecked] = 'X'+wv
                    continue
                if wv is None:
                    v['ox'][nChecked] = 'Xanswer is None'
                    continue
                
                ans2match=ans0[1]
                if isinstance(ans2match, dict) and ('ret' in ans2match): # 코드제출
                    if isinstance(v['py'][nChecked][2], str):
                        v['ox'][nChecked] = self.compare(v['py'][nChecked][2], ans2match['ret'])
                    else:
                        try:
                            wv=evalPyCode(ans2match, v['py'][nChecked][2])
                        except Exception as err:
                            v['ox'][nChecked]='X'+f'Error({err})'
                            continue
                        v['ox'][nChecked] = self.compare(wv, ans2match['ret'])

                elif isinstance(ans2match, str) and ans2match.startswith('code:'):
                    if self.checkAnswerCode(wv, ans2match.split('code:')[-1]): # 반환값: T/F 2023-04-15 self.추
                        v['ox'][nChecked] = 'O'
                    else:
                        v['ox'][nChecked] = 'Xincorrect arb. value'

                else:
                    v['ox'][nChecked] = self.compare(wv, ans2match)

            v['score']=sum(10 if c[0]=='O' else 5 if c[0]=='5' else 3 if c[0]=='3' else 0 for c in v['ox'])


    def saveResults(self):
        QG=self.QGs
        sheets=self.Sheets
        path4files=self.path4files
        WN=self.WN
        CAR={q:[0,0] for q in range(len(QG))} # 질문지에 사용된 질문-군 목록
        log={}

        for sid,v in sheets.items():
            if not v['file']: continue
            for ox, q in zip(v['ox'], v['orders']):
                CAR[q][0] +=1
                if ox=='O': CAR[q][1] += 1
            if v['log']:
                log[sid]=v['log']

        YESCAR=False if any(not carV[0] for carV in CAR.values()) else True

##        # 예전에 저장한 파일 이름 변경...    
##        if os.path.exists(os.path.join(path4files,f'{WN}Scores.txt')):
##            fileList=sorted([f for f in os.listdir(path4files) if 'Scores' in f], reverse=True)
##            for f in fileList:
##                if '_' in f:
##                    front, rear=f.split('_')
##                    rear, ext=rear.split('.')
##                    newf=front+f'_{int(rear)+1:02d}.'+ext
##                else:
##                    front, ext=f.split('.')
##                    newf=front+'_01.'+ext
##                os.rename(os.path.join(path4files,f), os.path.join(path4files,newf))

        try:
            with open(path4files+'\\'+WN+'Scores.txt','w') as f:
                for sid, v in sheets.items():
                    if v['msg']:
                        f.write(sid+f'__{v["msg"]}\n')
                    else:
                        f.write(sid+f'__{v["ox"]}({v["score"]})\n')
                        for jj, ox in enumerate(v['ox']):
                            if ox == 'X':
                                f.write(f'\t{jj+1}: ({ox})\n')
                
                if log:
                    f.write('\n---Error messages---\n')
                    for k in sorted(log.keys()):
                        f.write(f'{k} -- {log[k]}\n')
 
                if CAR and YESCAR:
                    f.write('\n---질문별 정답율---\n')
                    for k in sorted(CAR.keys()):
                        if CAR[k][0]:
                            f.write(f'{k} -- {CAR[k]}, {round(CAR[k][1]*100/CAR[k][0],1)}\n')
                        else:
                            f.write(f'{k} -- {CAR[k]}\n')


        except PermissionError:
            return 'Error -- text file is open.\nPlease close the file and try again'
        except Exception as err:
            return f'Error({err}) in Saving scores.', CAR
        

        try:
            import csv
            with open(path4files+'\\'+WN+'Scores.csv','w', newline='') as f:
                csvwriter=csv.writer(f, delimiter=',')
                for sid, v in sheets.items():
                    if v['msg']:
                        csvwriter.writerow([sid,v['msg'],0])
                    else:
                        csvwriter.writerow([sid, ''.join([s[0] for s in v["ox"]]), v["score"]])
            
            return 'Saving the scores to csv file is OK', CAR
        except PermissionError:
            return 'Error -- csv file is open.\nPlease close the file and try again', CAR
        except Exception as err:
            return f'Error in Saving csv file\nwith {err}\n@ {item}'
            

    @staticmethod
    def compare(A, B): # self.scoreThem에서 사용
        '''A: 학생답, B: 정답'''
        if A is None:
            return 'XNo answer'
        elif type(A) == type(B):
            if A == B:
                return 'O'
            elif isinstance(B, float):
                if round2MSF(A) == round2MSF(B):
                    return 'O'
                else:
                    #print(round2MSF(A), round2MSF(B))
                    return 'Xrounding error'
            else:
                #print(f'학생답: ++{A}++', type(A)) # 2023-04-14 added
                #print(f'정답: ++{B}++', type(B)) # 2023-04-14 added
                return 'Xwrong Answer'
        else:
            return 'Xwrong type'


    @staticmethod
    def checkAnswerCode(StudentValue, ansCode):  # 2023-04-14 added incomplete
        try:
            tmp=eval(ansCode)
        except Exception:
            tmp=False
        return tmp




import random, re, os, pickle, copy, webbrowser
from .makeChoices import generateChoices



class work():
    def __init__(self, name: str ='Preview', heading: str='Example', STDs: dict={'12345678': '홍 길동(abc def)'},
                 QGs: list=[], flagPreview: bool=True, flagChoice: bool=False, flagShuffling: bool=False,
                 resources: dict={}, flagPrevious: bool=False):
        self.Name=name
        self.Heading=heading
        self.STDs=STDs
        self.QGs=QGs
        self.Flag4Preview = flagPreview
        self.Flag4Choice = flagChoice
        self.Flag4Shuffling = flagShuffling
        #self.Flag4Answer = True
        self.Sheets={}
        self.resources = resources
        self.Flag4Previous=flagPrevious  # 2023-04-16 0.026

        self.prevWork=None  # 2023-04-16 0.026
        if self.Flag4Previous:
            fn=os.path.join('.',f'{self.Name}',f'{self.Name}Work.pickle')
            if os.path.exists(fn):
                self.prevWork=pickle.load(open(fn,'rb'))
                if any([self.prevWork.STDs != self.STDs, self.prevWork.QGs != self.QGs,
                        self.prevWork.Flag4Choice != self.Flag4Choice,
                        self.prevWork.Flag4Shuffling != self.Flag4Shuffling]):
                    self.prevWork=None


        if self.QGs and self.STDs:
            self.initializeWork()
            self.makeSheets()
            self.mkHTMLs()
            if not self.Flag4Preview:
                print(f'Question sheets in HTML files and answer files are saved in the "{self.Name}" folder')
                self.saveWork()
            
        
    def initializeWork(self):
        if not self.QGs or not self.STDs: return
        self.Sheets.clear()

        # check STDs
        if not isinstance(self.STDs, dict):
            print('STDs is not a proper dictionary.')
            return

        for k, v in self.STDs.items():
            if not k or not v:
                print('One of keys and values is not a proper value')
                return

            if not isinstance(k, str) or not isinstance(v, str):
                print('One of keys and values is not a string')
                return
                
        if self.Flag4Preview:
            for j, qg in enumerate(self.QGs):
                indices=list(range(len(qg[0])))
                self.Sheets[j]={'orders':indices,
                                  'seed':random.sample(range(10001,100000), len(qg[0]))}

        else:
            if '12345678' in self.STDs and self.STDs['12345678'] == '홍 길동(abc def)':
                print('STDs is not ready for question sheets')
                return

##            indices=list(range(len(self.QGs)))  2023-04-16 0.026
##            for std in self.STDs.keys():
##                self.Sheets[std]={'orders':indices.copy(),
##                                  'seed':random.sample(range(10001,100000), len(self.QGs))}
##
##            if self.Flag4Shuffling:
##                for v in self.Sheets.values():
##                    random.shuffle(v['orders'])

            if self.prevWork:
                for std in self.STDs.keys():
                    self.Sheets[std]={'orders': self.prevWork.Sheets[std]['orders'],
                                      'seed': self.prevWork.Sheets[std]['seed']}
            else:
                indices=list(range(len(self.QGs)))
                for std in self.STDs.keys():
                    self.Sheets[std]={'orders':indices.copy(),
                                      'seed':random.sample(range(10001,100000), len(self.QGs))}

                if self.Flag4Shuffling:
                    for v in self.Sheets.values():
                        random.shuffle(v['orders'])



    def makeSheets(self):
        if not self.QGs or not self.STDs: return
        
        if self.Flag4Preview:
            for std, v in self.Sheets.items():
                tmp=[]
                for seed, j in zip(v['seed'], v['orders']):
                    out=self.getQA(self.QGs[std], j, seed, self.resources) #<--- 아래와 차이, 2023.04-15 추가
                    if out[0] is None:
                        print(std, out, self.QGs[cat][name].Name, '????---Try again!')
                        self.Sheets.clear()
                        return

##                    if self.QGs[std][-1] == 'python': #<--- 아래와 차이
##                        tmp.append(out)
##                    elif isinstance(out[1], (int, float)) and self.Flag4Choice:
##                        xtra, ansChoice, ans, choicesUsed = generateChoices(out[1])
##                        out = (out[0]+xtra, int(ansChoice), out[-1], (origAnswer, choicesUsed))
##                        tmp.append(out)
##                    elif isinstance(out[1], dict) and 'choices' in out[1]:
##                        xtra, ansChoice, ans, choicesUsed = generateChoices(out[1])
##                        out = (out[0]+xtra, int(ansChoice), out[-1], (origAnswer, choicesUsed))
##                        tmp.append(out)
##                    else:
##                        tmp.append(out)
                    tmp.append(self.out2out(self.QGs[std][-1], out))

                v['Q&A'] = tmp
                
        else:
            for std, v in self.Sheets.items():
                tmp=[]
                for seed, j in zip(v['seed'], v['orders']):
                    out=self.getQA(self.QGs[j], -1, seed, self.resources) #<--- 위와 차이, 2023.04-15 추가
                    if out[0] is None:
                        print(std, out, self.QGs[cat][name].Name, '????---Try again!')
                        self.Sheets.clear()
                        return

##                    if self.QGs[j][-1] == 'python': #<--- 위와 차이
##                        tmp.append(out)
##                    elif isinstance(out[1], (int, float)) and self.Flag4Choice:
##                        xtra, ansChoice, ans, choicesUsed = generateChoices(out[1])
##                        out = (out[0]+xtra, int(ansChoice), out[-1], (origAnswer, choicesUsed))
##                        tmp.append(out)
##                    elif isinstance(out[1], dict) and 'choices' in out[1]:
##                        xtra, ansChoice, ans, choicesUsed = generateChoices(out[1])
##                        out = (out[0]+xtra, int(ansChoice), out[-1], (origAnswer, choicesUsed))
##                        tmp.append(out)
##                    else:
##                        tmp.append(out)
                    tmp.append(self.out2out(self.QGs[j][-1], out))

                v['Q&A'] = tmp


    def out2out(self, qType, out):
        if qType == 'python':
            return out
        elif isinstance(out[1], (int, float)) and self.Flag4Choice:
            origAnswer=copy.deepcopy(out[1])
            xtra, ansChoice, ans, choicesUsed = generateChoices(out[1])
            return (out[0]+xtra, int(ansChoice), out[-1], (origAnswer, choicesUsed))
        elif isinstance(out[1], dict) and 'choices' in out[1]:
            origAnswer=copy.deepcopy(out[1])
            xtra, ansChoice, ans, choicesUsed = generateChoices(out[1])
            return (out[0]+xtra, int(ansChoice), out[-1], (origAnswer, choicesUsed))
        else:
            return out
        



    def getAnswers(self):
        return {std:[item[1] for item in v['Q&A']] for std,v in self.Sheets.items()}


    def saveWork(self):
        if self.Flag4Preview:
            print("'Preview' flag is True, ignoring calls to saveWork(). Change the Flag4Preview to False and try again")
            return
        
        if not os.path.exists(os.path.join('.', f'{self.Name}')):
            os.mkdir(os.path.join('.', f'{self.Name}'))

        # save work to a pickle file - to be used to score student's answer files
        pickle.dump(self, open(os.path.join('.',f'{self.Name}',f'{self.Name}Work.pickle'),'wb'),
                               protocol=pickle.HIGHEST_PROTOCOL)

        # save all answers to a text file - to provide to students after scoring is done
        with open(os.path.join('.',f'{self.Name}',f'{self.Name}Answers.txt'), mode='w', encoding='utf-8') as f:
            f.write(f'{self.Heading} Answers {self.Name}\n')
            for SID, ans in self.getAnswers().items():
                f.write('SID: '+SID+'\n')
                for j, item in enumerate(ans):
                    f.write(f'\t{j+1}: {item}\n')


    @staticmethod
    def getQA(QG, indxJ: int =-1, seed: int =None, resources: dict=None):
        if seed:
            random.seed(seed)

        Q, strA, _, qType = QG

        if indxJ<0:
            indxJ=random.choice(range(len(Q)))

        try:
            exec(strA)
        except Exception as err:
            return f'<font color="#dd0000"><bold>Error</bold></font>::exec(A)...{err}', None, None, None

        try:
            dataA=eval('data')
        except Exception as err:
            return f'<font color="#dd0000"><bold>Error</bold></font>::eval(data)....{err}', None, None, None
        
        try:
            answers=eval('answer')
        except Exception as err:
            return f'<font color="#dd0000"><bold>Error</bold></font>::eval(answer)....{err}', None, None, None

        if not answers:
            return f'<font color="#dd0000"><bold>Error</bold></font>::empty answer. Add answer.', None, None, None

        if len(answers)<len(Q):
            return f'<font color="#dd0000"><bold>Error</bold></font>::insufficent answers. Add answer.', None, None, None

        strQ=Q[indxJ]

        if '<!--' in strQ:
            indx=strQ.index('<!--')
            strQ = strQ[:indx]

        if '{%' in strQ and '%}' in strQ:
            for string in re.findall(r"\{%(.*?)\%}", strQ):
                try:
                    strQ=strQ.replace('{%'+string+'%}', str(eval(string)))
                except Exception as err:
                    return f'<font color="#dd0000"><bold>Error</bold></font>::질문 #{indxJ+1}에 들어있는 변수-{string}....{err}', None, None, None

        if dataA:
##            if strQ.rstrip().endswith('</pre>'):
##                strQ += '<p><b>DATA:</b><br>'+'<br>'.join(f'{v}' for v in dataA)+'</p>'
##            else:
##                strQ += '<p><b>DATA:</b><br>'+'<br>'.join(f'{v}' for v in dataA)+'</p>'
            strQ += '<pre><b>DATA:</b><br>'+'<br>'.join(f'{v}' for v in dataA)+'</pre>'

        return strQ, answers[indxJ], indxJ, qType


    def mkHTMLs(self):
        if not self.Sheets: return 'No work is given.'

        HTMLfiles={}
        for std, v in self.Sheets.items():
            qna=v['Q&A']
            list4HTML=[(j, item[0].split('%HTML%')[-1]) for j, item in enumerate(qna) if '%HTML%' in item[0]]
            list4HTMLindices=[j for j, item in list4HTML]

            if list4HTML: # 단순한 설명 html 내용이 있는 경우
                func='<script type="application/javascript">\n'
                for j, item in enumerate(qna):
                    if j in list4HTMLindices: # 단순한 설명 html인 경우
                        func +=templateF.replace('qNUM', f'{j}').replace('TEXT', '')
                    else: # 질문인 경우
                        func += self.mkHtmlFuncDisplayQ(j, item[0], None, self.resources)
                        
            else: # 예전과 동일
                func='<script type="application/javascript">\n'
                for j, item in enumerate(qna):
                    if isinstance(item[-1], (tuple, list)) and len(item[-1])>1 and isinstance(item[-1][0], dict) and 'cols' in item[-1][0]:
                        func += self.mkHtmlFuncDisplayQ(j, item[0], item[-1], self.resources) # 2023-04-03 보기문항이 그림들인 경우
                    else:
                        func += self.mkHtmlFuncDisplayQ(j, item[0], None, self.resources) # 예전과 동일
                
            func += templateEnd.replace('QMAX', f'{len(qna)-1}')

            if self.Flag4Preview:
                table=templateTable+''.join([f'\n{jj+1}: {qna[jj][1]}' for jj in range(len(qna))])
                table=table.replace('NAME', f'{std}') #<---
            else:
                table=templateTable+''.join([f'\n{jj+1}: ' for jj in range(len(qna))])
                table=table.replace('NAME', self.STDs[std])
            
            table += '</textarea></td></tr></tbody></table>'

            if self.Flag4Preview:
                file = templateHEAD.replace('TITLE', f'{self.Heading} {self.Name}{std} {std}') + table
            else:
                file = templateHEAD.replace('TITLE', f'{self.Heading} {self.Name}{std} {self.STDs[std]}') + table

            file = file.replace('ONCONTEXTMENU', '<body style="word-break:keep-all; display: block; font-family: malgun;  font-size: 18px;">')

            for jj in range(len(qna)):
                file += f'<input type="button" onclick="javascript:display_Q{jj}();" class= "btn" value="Q-{jj+1}"  />\n'

            file += templateBodyR.replace('FILENAME', f'{self.Name}{std}')

            if list4HTML:
                for j, txt in list4HTML:
                    file += f'<div id="HTML{j}" style="word-break:keep-all; padding: 0px 20px 10px 20px; background-color: #f2f6fc; border: 1px solid #999; display: block; font-family: malgun;  font-size: 18px;line-height: 1.5em; display=none;">'
                    file += txt
                    file += '</div>'
            
            if 'line_arrow' in func:
                func = line_arrow +func


            if self.Flag4Preview:
                file += JS4TABLEwithRadiobuttons.replace('FILENAME', f'{self.Name}{std}').replace('NAME', f'{std}')
            else:
                file += JS4TABLEwithRadiobuttons.replace('FILENAME', f'{self.Name}{std}').replace('NAME', self.STDs[std])
            
            HTMLfiles[std]=file+func

        if not os.path.exists(os.path.join('.', f'{self.Name}')):
            os.mkdir(os.path.join('.', f'{self.Name}'))
        if not os.path.exists(os.path.join('.', f'{self.Name}', f'{self.Name}')):
            os.mkdir(os.path.join('.', f'{self.Name}', f'{self.Name}'))

        for k, v in HTMLfiles.items():
            open(os.path.join('.',f'{self.Name}',f'{self.Name}',f'{self.Name}_{k}.html'),
                 mode='w', encoding='utf-8').write(v)

        # index.html
        if self.Flag4Preview:
            file = templateBodyF.replace('TITLE', f'{self.Heading} {self.Name}') + '\n<table><tbody>'
            for k in self.Sheets.keys():
                file += f'\n<tr><td><a href="./{self.Name}_{k}.html">{k}--{self.QGs[k][2]}</a></td></tr>'
            file +='</tbody></table></html>'
            open(os.path.join('.',f'{self.Name}',f'{self.Name}','index.html'), mode='w', encoding='utf-8').write(file)
            if k:
                webbrowser.open(os.path.join('.',f'{self.Name}',f'{self.Name}','index.html'))
            else:
                webbrowser.open(os.path.join('.',f'{self.Name}',f'{self.Name}',f'{self.Name}_{k}.html'))
        else:
            file = templateBodyF.replace('TITLE', f'{self.Heading} {self.Name}') + '\n<table><tbody>'
            for k, Sname in self.STDs.items():
                file += f'\n<tr><td><a href="./{self.Name}_{k}.html">{k}--{Sname}</a></td></tr>'
            file +='</tbody></table></html>'
            open(os.path.join('.',f'{self.Name}',f'{self.Name}','index.html'), mode='w', encoding='utf-8').write(file)


    @staticmethod
    def mkHtmlFuncDisplayQ(qJ, txt, xtra=None, resources=None): # 2023-04-03 correctAnswer --> xtra
        if '<!--' in txt: # 2023-03-21 추가
            indx=txt.index('<!--')
            txt = txt[:indx]
        
        txt=txt.strip().replace('\n', '')  # '\n' 없애야 하는 이유는 FIG% %FIG 을 사용하기 위해...그리고 ... 많음.

        str2bUsed=''
        if 'init(' in txt:
            init=txt.split('init(')[-1].split(';')[0]
            str2bUsed=f'init({init};'
            txt = txt.replace(str2bUsed,'')

        figObjts=[]
##        if 'FIG%' in txt: # 직접 변경 가능
##            for item in re.findall(figPattern, txt):
##                itemStripped=item.strip()
##                if itemStripped:
##                    figObjts.append(itemStripped)
##                    txt=txt.replace(f'FIG%{item}%FIG', '')
##
        if 'figure(' in txt: # 한 번 만든 것.. 직접 변경 불가능..
            for item in re.findall(figurePattern, txt):
                figureKey=item.strip()
                if figureKey and figureKey in resources and resources[figureKey] and resources[figureKey] != 'HTTP Error 404: Not Found':
                    figObjts.append(resources[figureKey])
                    txt=txt.replace(f'figure({item})','')
                else:
                    txt=txt.replace(f'figure({item})',f'figure({item}):<font color="#dd0000">HTTP Error 404: Not Found</font>')

        txt=txt.replace("'", "&#39;") # 2022.08.07 둘다 사용하지 않는 것으로...

        if 'chr(9)' in txt:
            txt=txt.replace('chr(9)','&nbsp;&nbsp;&nbsp;&nbsp;')

        if 'EQ%' in txt:
            for item in re.findall(eqPattern, txt):
                txt=txt.replace(f'EQ%{item}%EQ',
                                r'<img src="http://latex.codecogs.com/png.latex?\dpi{130}'+ f' {item}"/>')
            txt=txt.replace('\\', '\\\\')

        if str2bUsed:
            for j, obj in enumerate(figObjts):
                if '//init();' in obj:
                    figObjts[j] = obj.replace('//init();', str2bUsed)

        txt = f'<p><font color="#dd0000"><strong>{qJ+1}.</strong></font> '+txt
        #indx = txt.find('<form')
        indx = txt.find('<div> ')  # 반드시 makeChoices._ChoicesWithRadiobuttons() 와 맞춰야 함....
        if indx == -1:
            txt += '</p>'
        else:
            txt = txt[:indx]+'</p>'+txt[indx:]


    ##    out=templateF.replace('TEXT', f'<p><font color=red><strong>{qJ+1}.</strong></font> '+txt).replace('qNUM', f'{qJ}')
        out=templateF.replace('TEXT', txt).replace('qNUM', f'{qJ}')

        if figObjts:
            if xtra:
                figureKey += 'F'
                jsCode=f'function {figureKey}'+cnvs2nd+figObjts[0]+'}'
                jsCode=jsCode.replace('//init();', 'init(prms);')

                more = [f'{figureKey}("cnvs{j}", {item});' for j, item in enumerate(xtra[1])]
                out=out.replace('JS_CODE', 'cnvs.height=0;\n'+jsCode+''.join(more))
                    
            else:
                out=out.replace('JS_CODE', 'cnvs.height=300;\n'+'\n'.join(figObjts))
        else:
            out=out.replace('JS_CODE', 'cnvs.height=0;')

        for item in re.findall('<br><br>*<br>', out):
            out=out.replace(item, '')

        return out

            


eqPattern='EQ%(.*?)%EQ'
figPattern='FIG%(.*?)%FIG'
figurePattern='figure\((.*?)\)'

cnvs2nd='''(cnvsName, prms) {
var cnvs = document.createElement("canvas");
div = document.getElementById(cnvsName);
div.appendChild(cnvs);
var ctx = cnvs.getContext("2d");
'''

# 단순한 html page 용 template
templateHEAD='''<!DOCTYPE html>
<html>
<head>
<title>TITLE</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<style>
.prePrompt {float: left; margin: 20px   0px 20px 10px; padding: 10px   5px 10px   5px; background: #efc; font-family: bold Courier New; font-size: 14px; line-height: 0.5 em; width: 30px;              text-align: left; border: 1px solid powderblue; border-right: none; color: #700;}
.preBody     {float: left; margin: 20px 10px 20px   0px; padding: 10px 10px 10px 10px; background: #efc; font-family: bold Courier New; font-size: 14px; line-height: 0.5 em; width: text-content; text-align: left; border: 1px solid powderblue; }
.preChoice  {float: left; margin: 0px 10px 20px 10px; padding: 10px; background: #fff; font-family: bold Courier New; font-size: 14px; line-height: 0.5 em; width: text-content; text-align: left; border: 1px solid powderblue; }
.btn             {align-items: center; appearance: none; background-color: #eef; border-radius: 24px; border-style: none; box-shadow: rgba(0, 0, 0, .2) 0 3px 5px -1px,rgba(0, 0, 0, .14) 0 6px 10px 0,rgba(0, 0, 0, .12) 0 1px 18px 0; box-sizing: border-box; color: #3c40e3; cursor: pointer; display: inline-flex; fill: currentcolor; font-family: "Google Sans",Roboto,Arial,sans-serif; font-size: 16px; font-weight: 500; height: 48px; justify-content: center; letter-spacing: .25px; line-height: normal; max-width: 100%; overflow: visible; padding: 2px 24px; margin: 5px; position: relative; text-align: center; text-transform: none; transition: box-shadow 280ms cubic-bezier(.4, 0, .2, 1),opacity 15ms linear 30ms,transform 270ms cubic-bezier(0, 0, .2, 1) 0ms; user-select: none; -webkit-user-select: none; touch-action: manipulation; width: auto; will-change: transform,opacity; z-index: 0;}
.btn2           {align-items: center; background-clip: padding-box; background-color: #fa6400; border: 1px solid transparent; border-radius: .25rem; box-shadow: rgba(0, 0, 0, 0.2) 0 1px 3px 0; box-sizing: border-box; color: #fff; cursor: pointer; display: inline-flex; font-family: system-ui,-apple-system,system-ui,"Helvetica Neue",Helvetica,Arial,sans-serif; font-size: 20px; font-weight: 600; justify-content: center; line-height: 1.25; margin: 0px 10px 2px 5px; min-height: 3rem; padding: calc(.875rem - 1px) calc(1.5rem - 1px); position: relative; text-decoration: none; transition: all 250ms; user-select: none; -webkit-user-select: none; touch-action: manipulation; vertical-align: baseline; width: auto;}

.radiocontainer {background-color:#e8e8f8; display: block; position: relative; padding:10px 20px 10px 50px; margin: 0px 5px 2px 0px; cursor: pointer; font-size: 18px; -webkit-user-select: none; -moz-user-select: none; -ms-user-select: none; user-select: none; word-wrap: break-word; min-width: 200px;}
.radiocontainer input {  position: absolute;  opacity: 0;  cursor: pointer;}
.checkmark {  position: absolute;  top: 15px;  left: 15px;  height: 19px;  width: 19px;  background-color: #fff;  border-radius: 50%;}
.checkedlabel {  background-color:#ddd;}
.radiocontainer:hover input ~ .checkmark {}
.radiocontainer:hover {  background-color: #ddd;}
.radiocontainer input:checked ~ .checkmark {  background-color: #12693F;}
.checkmark:after {  content: "";  position: absolute;  display: none;}
.radiocontainer input:checked ~ .checkmark:after {  display: block;}
.radiocontainer .checkmark:after {top: 6px; left: 6px; width: 7px; height: 7px; border-radius: 50%; background: white;}
</style>
</head>
ONCONTEXTMENU
<center><h4>TITLE</h4></center>'''

templateBodyF='''<!DOCTYPE html>
<html>
<head>
<title>TITLE</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<body oncontextmenu="return false">
<center><h4>TITLE</h4></center>'''

templateBodyR='''<hr> <div id="Qs" style="word-break:keep-all; display: block; font-family: malgun; font-size: 18px; margin: 0px 10px 0px 10px; padding: 0px 5px 0px 5px;"></div>
<canvas id="canvas" width="800" height="0"></canvas>
'''

# backtick `    python의 '''와 동일
templateF='''function display_QqNUM() {
document.getElementById("Qs").innerHTML=`TEXT`;
Qnumber = qNUM;
prmtrQ = qNUM;
setCheckedRadio(qNUM);

var cnvs = document.getElementById("canvas");
var ctx = cnvs.getContext("2d");
ctx.clearRect(0,0, cnvs.width, cnvs.height);
JS_CODE
}
'''


templateF_000='''function display_QqNUM() {
document.getElementById("Qs").innerHTML='TEXT';
Qnumber = qNUM;
prmtrQ = qNUM;
setCheckedRadio(qNUM);

var cnvs = document.getElementById("canvas");
var ctx = cnvs.getContext("2d");
ctx.clearRect(0,0, cnvs.width, cnvs.height);
JS_CODE
}
'''




templateEnd='''
document.onkeydown = function (e) {
  if (e.altKey && e.keyCode == 39) {
    Qnumber += 1;
    if (Qnumber>Qmax) {Qnumber = 0};
    eval("display_Q"+Qnumber+"()");}
  else if (e.altKey && e.keyCode == 37) {
    Qnumber -= 1;
    if (Qnumber<0) {Qnumber = Qmax};
    eval("display_Q"+Qnumber+"()");}
}
var Qnumber = 0;
var Qmax = QMAX;
var answers = [];
var prmtrs = [];
var prmtrQ = null;
for (let j=0; j<=Qmax; j++) {answers.push(null);prmtrs.push(null);}

display_Q0();</script>\n</body>\n</html>
'''

templateTable='''<ul style="font-size: 14px">
<li>객관식 질문의 답은 정수로 답하시오. For multiple-choice questions, answer with an integer.</li>
<li>단답형 질문의 숫자 답은, 가장 중요한 3개의 숫자로 반올림하시오. For numerical answer to short questions, round to the three most significant digits.</li>
<li>요청에 따라 숫자를 문자열로 제출해는 경우, 양쪽에 따옴표를 붙이시오. If you are asked to submit a number as a string, put quotes around it.</li></ul>
<table>
<tbody>
<tr>
<td ><button onclick="saveTextAsFile()" class="btn2">Save<br>Answers<br>to File</button></td>
<td><textarea id="inputTextToSave" cols="155" rows="5" style="font-family: malgun;  font-size: 16px; padding: 5px  0px  5px  20px; ">
나, NAME은(는) 위 내용을 읽고 아래 답을 직접 작성했음을 확인합니다. I, NAME, confirm that I read the notes above and wrote the answer myself below.
--------------------------------------------------------------------------------------------------------------------'''


JS4TABLEwithRadiobuttons='''<script type="text/javascript">
function saveTextAsFile() {
  var textToSave = document.getElementById("inputTextToSave").value;
  var textToSaveAsBlob = new Blob([textToSave], {type:"text/plain"});
  var textToSaveAsURL = window.URL.createObjectURL(textToSaveAsBlob);
  var fileNameToSaveAs = "FILENAME.txt"

  var downloadLink = document.createElement("a");
  downloadLink.download = fileNameToSaveAs;
  downloadLink.innerHTML = "Download File";
  downloadLink.href = textToSaveAsURL;
  downloadLink.onclick = destroyClickedElement;
  downloadLink.style.display = "none";
  document.body.appendChild(downloadLink);
  downloadLink.click();
}

function addAnswers2Textarea() {
  const textarea = document.getElementById("inputTextToSave");
  var text = textarea.value;
  const searchTerm = "--------------------------------------------------------------------------------------------------------------------";
  const indexOfFirst = text.indexOf(searchTerm);
  var newText = text.slice(0, indexOfFirst+searchTerm.length);
  for (let q=0; q<=Qmax; q++) {
    if (answers[q] == null) {newText += String.fromCharCode(13)+(q+1).toString()+": ";}
    else {newText += String.fromCharCode(13)+(q+1).toString()+": "+answers[q].toString();}
  }
  textarea.value = newText;
}

function handleClick(myRadio) {
  answers[parseInt(myRadio.name.slice(1))]=parseInt(myRadio.value)+1;
  addAnswers2Textarea();
}

function setCheckedRadio(q) {
  var radios = document.getElementsByName('Q'+q.toString());
  //for (let j=0; j<radios.length; j++) {radios[j].removeAttribute("checked");}
  if (radios.length > 0 && answers[q] != null) {radios[answers[q]-1].setAttribute("checked", true);}
}


function destroyClickedElement(event) {
    document.body.removeChild(event.target);
}
</script>
'''


line_arrow='''<script type="application/javascript">
function line_arrow(context, fromx, fromy, tox, toy, color, end) {
  var headlen = 7;
  var dx = tox - fromx;
  var dy = toy - fromy;
  var angle = Math.atan2(dy, dx);
  context.beginPath();
  context.strokeStyle = color;
  if (end == "last") {
    context.moveTo(fromx, fromy);
    context.lineTo(tox, toy);
    context.lineTo(tox - headlen * Math.cos(angle - Math.PI / 6), toy - headlen * Math.sin(angle - Math.PI / 6));
    context.moveTo(tox, toy);
    context.lineTo(tox - headlen * Math.cos(angle + Math.PI / 6), toy - headlen * Math.sin(angle + Math.PI / 6));
  }
  else if (end=="both") {
    context.moveTo(fromx, fromy);
    context.lineTo(tox, toy);
    context.lineTo(tox - headlen * Math.cos(angle - Math.PI / 6), toy - headlen * Math.sin(angle - Math.PI / 6));
    context.moveTo(tox, toy);
    context.lineTo(tox - headlen * Math.cos(angle + Math.PI / 6), toy - headlen * Math.sin(angle + Math.PI / 6));
    context.moveTo(fromx, fromy);
    context.lineTo(fromx + headlen * Math.cos(angle + Math.PI / 6), fromy + headlen * Math.sin(angle + Math.PI / 6));
    context.moveTo(fromx, fromy);
    context.lineTo(fromx + headlen * Math.cos(angle - Math.PI / 6), fromy + headlen * Math.sin(angle - Math.PI / 6));
  }
  else if (end == "first"){
    context.moveTo(tox, toy);
    context.lineTo(fromx, fromy);
    context.lineTo(fromx + headlen * Math.cos(angle + Math.PI / 6), fromy + headlen * Math.sin(angle + Math.PI / 6));
    context.moveTo(fromx, fromy);
    context.lineTo(fromx + headlen * Math.cos(angle - Math.PI / 6), fromy + headlen * Math.sin(angle - Math.PI / 6));
  }
  context.stroke();
}
</script>
'''
